# This module is used to generate ip addresses of a given range

import ipaddress


class IpGeneration:
    """ Class for generation of range of ip addresses """

    def __init__(self, input_ip):
        self._input_ip = input_ip
        self._ip_value = None
        self._data = []

    def check(self, network):
        """ Method to verify IP address network
        :param: user entered ip address
        :param: bool obj for Ip version
        :returns: IP address value or none in case of failure
        """
        if network:
            grp = ipaddress.IPv4Network
        else:
            grp = ipaddress.IPv6Network
        try:
            self._ip_value = grp(self._input_ip)
        except (ipaddress.AddressValueError, ValueError, ipaddress.NetmaskValueError):
            return
        return

    def generate_ip(self):
        """ Methods generate an itr for a range of ip address
        :param: user input ipaddress
        :returns: Range of ip address and None in case of incorrect input
        """
        self.check(True)
        if self._ip_value is None:
            self.check(False)

        if self._ip_value is None:
            print("Incorrect Network IP entered")
            return None

        for ipaddr in self._ip_value:
            self._data.append(ipaddr)

        return self._data


if __name__ == '__main__':
    for ip_address in IpGeneration('192.168.0.0/24').generate_ip():
        print(ip_address)



# Module for implement TCP server

import socketserver


class server_handler(socketserver.BaseRequestHandler):
    """ The request handler class for the server """

    def handle(self):
        """Method to echo user response"""
        print("Enter anything")
        self._data = self.request.recv(1024).decode("utf-8")
        print("{} wrote: ".format(self.client_address[0]))
        print(self._data)
        self.request.sendall(bytes(self._data, 'utf-8'))


def creating_tcp_server(host='localhost',port=9999):
    """ Method to activate tcp server
    :param: host: Host to connect
    :param: port: port to connect
    """
    with socketserver.TCPServer((host, port), server_handler) as server:
        server.handle_request()


if __name__ == "__main__":
    creating_tcp_server()



import socket


class ServerLogin:
    """ Class to create a server login protocol """
    def __init__(self):
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._conn, self._addr = None, None
        self._username, self._password = None, None

    def creating_server(self, host, port):
        """ Method to activate server and login into it
        :param host: host to connect
        :param port: port number to connect
        """
        with self._sock:
            self._sock.bind((host, port))
            self._sock.listen(1)
            self._conn, self._addr = self._sock.accept()
            with self._conn:
                print('Connected by', self._addr)
                self._conn.sendall(b"Enter username")
                self._username = self._conn.recv(1024)
                self._conn.sendall(b"Enter password")
                self._password = self._conn.recv(1024)
                if self._username == b"Rishabh" and self._password == b"Pallod":
                    self._conn.sendall(b"Login successful")
                else:
                    self._conn.sendall(b'Incorrect user details')
        return


if __name__ == "__main__":
    ServerLogin().creating_server('localhost', 50007)



# This module is used to remotely execute server functions

import xmlrpc.client


class ExecRemote:
    """ Class to remotely run floor divide function on a server """
    def __init__(self):
        self._server_proxy = None

    def remote_function(self, server_url, a, b=2):
        """ Method to remotely access raise power function on the server
            :arg: server_url : Takes the host and port details of server
            :arg: a, b : int objs as input for floor_divide method

            :returns: int obj on success and None on failure
        """
        self._server_proxy = xmlrpc.client.ServerProxy(server_url)
        try:
            return self._server_proxy.floor_divide(a, b)
        except xmlrpc.client.Fault as err:
            print("A fault occurred")
            print("Fault code: %d" % err.faultCode)
            print("Fault string: %s" % err.faultString)
            return None


if __name__ == '__main__':
    print(ExecRemote().remote_function("http://localhost:8000/", 5, 3))
